@@ -1,6 +1,6 @@
 #   Permute.pm
 #
-#   Copyright (c) 1999,2000 Edwin Pratomo
+#   Copyright (c) 1999,2000,2001 Edwin Pratomo
 #
 #   You may distribute under the terms of either the GNU General Public
 #   License or the Artistic License, as specified in the Perl README file,
@@ -17,11 +17,13 @@ require DynaLoader;
 require AutoLoader;
 
 @ISA = qw(Exporter DynaLoader);
+@EXPORT_OK = qw(permute);
+
 # Items to export into callers namespace by default. Note: do not export
 # names by default without a very good reason. Use EXPORT_OK instead.
 # Do not simply export all your public functions/methods/constants.
 
-$VERSION = '0.02';
+$VERSION = '0.03';
 
 bootstrap Algorithm::Permute $VERSION;
 
@@ -40,11 +42,14 @@ Algorithm::Permute - Handy and fast permutation with object oriented interface
 
   use Algorithm::Permute;
 
-  $p = new Algorithm::Permute(['a'..'d']);
+  my $p = new Algorithm::Permute(['a'..'d']);
   while (@res = $p->next) {
     print join(", ", @res), "\n";
   }
 
+  my @array = (1..9);
+  Algorithm::Permute::permute { print "@array\n" } @array;
+
 =head1 DESCRIPTION
 
 This handy module makes performing permutation in Perl easy and fast, 
@@ -81,27 +86,63 @@ entire set has been produced or not. Has no useful return value.
 
 =back
 
+=head1 CALLBACK STYLE INTERFACE
+
+Starting with version 0.03, there is a function - not exported by
+default - which supports a callback style interface:
+
+=over 4
+
+=item permute BLOCK ARRAY
+
+A block of code is passed, which will be executed for each permutation. The array will be changed in place,
+and then changed back again before C<permute> returns. During the execution of the callback,
+the array is read-only and you'll get an error if you try to change its length. (You I<can>
+change its elements, but the consequences are liable to confuse you and may change in future
+versions.)
+
+You have to pass an array, it can't just be a list. It B<does> work with special arrays
+and tied arrays, though unless you're doing something particularly abstruse you'd be
+better off copying the elements into a normal array first. Example:
+
+ my @array = (1..9);
+ permute { print "@array\n" } @array;
+
+The code is run inside a pseudo block, rather than as a normal subroutine. That means
+you can't use C<return>, and you can't jump out of it using C<goto> and so on. Also,
+C<caller> won't tell you anything helpful from inside the callback. Such is the price
+of speed.
+
+The order in which the permutations are generated is not guaranteed, so don't rely
+on it. 
+
+The low-level hack behind this function makes it currently the fastest way of doing
+permutation among others. 
+
+=back
+
 =head1 COMPARISON
 
 I've collected some Perl routines and modules which implement permutation,
-and do some simple benchmark. The result, which is of course predictable, 
-and obvious, is the following.
+and do some simple benchmark. The whole result is the following.
 
 Permutation of B<eight> scalars:
 
- Abigail's: 14 wallclock secs (13.21 usr +  0.49 sys = 13.70 CPU)
- Algorithm::Permute: 3 wallclock secs ( 2.96 usr +  0.02 sys =  2.98 CPU)
- List::Permutor:  9 wallclock secs ( 8.98 usr +  0.02 sys =  9.00 CPU)
-    MJD's: 56 wallclock secs (55.54 usr +  0.18 sys = 55.72 CPU)
- perlfaq4: 65 wallclock secs (64.71 usr +  0.22 sys = 64.93 CPU)
+ Abigail's:  9 wallclock secs ( 8.07 usr +  0.30 sys =  8.37 CPU)
+Algorithm::Permute:  5 wallclock secs ( 5.72 usr +  0.00 sys =  5.72 CPU)
+Algorithm::Permute qw(permute):  2 wallclock secs ( 1.65 usr +  0.00 sys =  1.65 CPU)
+List::Permutor: 27 wallclock secs (26.73 usr +  0.01 sys = 26.74 CPU)
+     MJD's: 32 wallclock secs (32.55 usr +  0.02 sys = 32.57 CPU)
+  perlfaq4: 36 wallclock secs (35.27 usr +  0.02 sys = 35.29 CPU)
 
 Permutation of B<nine> scalars (the Abigail's routine is commented out, because
 it stores all of the result in memory, swallows all of my machine's memory):
 
- Algorithm::Permute: 14 wallclock secs (14.13 usr + 0.07 sys = 14.20 CPU)
- List::Permutor: 67 wallclock secs (65.78 usr +  0.47 sys = 66.25 CPU)
-    MJD's: 530 wallclock secs (516.57 usr +  2.10 sys = 518.67 CPU)
- perlfaq4: 498 wallclock secs (490.49 usr +  1.65 sys = 492.14 CPU)
+ Algorithm::Permute: 43 wallclock secs (42.93 usr +  0.04 sys = 42.97 CPU)
+ Algorithm::Permute qw(permute): 15 wallclock secs (14.82 usr +  0.00 sys = 14.82 CPU)
+ List::Permutor: 227 wallclock secs (226.46 usr +  0.22 sys = 226.68 CPU)
+     MJD's: 307 wallclock secs (306.69 usr +  0.43 sys = 307.12 CPU)
+  perlfaq4: 272 wallclock secs (271.93 usr +  0.33 sys = 272.26 CPU)
 
 The benchmark script is included in the bench directory. I understand that 
 speed is not everything. So here is the list of URLs of the alternatives, in 
@@ -110,17 +151,21 @@ case you hate this module.
 =over 4
 
 =item * 
+
 Mark Jason Dominus' technique is discussed in chapter 4 Perl Cookbook, so you 
 can get it from O'Reilly: 
 ftp://ftp.oreilly.com/published/oreilly/perl/cookbook
 
 =item *
+
 Abigail's: http://www.foad.org/~abigail/Perl
 
 =item *
+
 List::Permutor: http://www.cpan.org/modules/by-module/List
 
 =item *
+
 The classic way, usually used by Lisp hackers: perldoc perlfaq4
 
 =back
@@ -129,6 +174,11 @@ The classic way, usually used by Lisp hackers: perldoc perlfaq4
 
 =over 4
 
+=item * 
+
+September 5, 2001 - version 0.03. A callback style interface, which is very
+fast, is added. 
+
 =item *
 
 September 2, 2000 - version 0.02. Major interface changes, now using object
@@ -145,7 +195,8 @@ October 3, 1999 - Alpha release, version 0.01
 =head1 AUTHOR
 
 Edwin Pratomo, I<ed.pratomo@computer.org>. The object oriented interface is
-taken from Tom Phoenix's C<List::Permutor>.
+taken from Tom Phoenix's C<List::Permutor>. Robin Houston
+<robin@kitsite.com> invented and contributed the callback style interface. 
 
 =head1 ACKNOWLEDGEMENT
 
